# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2020-02-21 11:27
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp', '0004_auto_20200221_1118'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ldpsource',
            options={'default_permissions': ('add', 'change', 'delete', 'view', 'control'), 'ordering': ('federation',)},
        ),
    ]
